@@ -78,20 +78,6 @@ int ve_private_is_ploop(const char *private)
 	return (stat_file(image) == 1);
 }
 
-int check_ploop_size(unsigned long size)
-{
-	const unsigned long max_size = /* 2T - 2G - 1M, in Kb */
-		(2ull << 30) - (2ull << 20) - (1 << 10);
-
-	if (size < max_size)
-		return 0;
-
-	logger(-1, 0, "Error: diskspace is set too big for ploop: %lu KB "
-			"(max size is %lu KB)", size, max_size - 1);
-
-	return -1;
-}
-
 #ifdef HAVE_PLOOP
 /* This should only be called once */
 static int load_ploop_lib(void)
@@ -217,6 +203,7 @@ int vzctl_mount_image(const char *ve_private, struct vzctl_mount_param *param)
 	mount_param.target = param->target;
 	mount_param.quota = param->quota;
 	mount_param.mount_data = param->mount_data;
+	mount_param.fsck = param->fsck;
 
 	PLOOP_CLEANUP(ret = ploop.mount_image(di, &mount_param));
 	if (ret) {
@@ -328,9 +315,6 @@ int vzctl_resize_image(const char *ve_private, unsigned long long newsize)
 		return VZ_VE_PRIVATE_NOTSET;
 	}
 
-	if (check_ploop_size(newsize) < 0)
-		return VZ_DISKSPACE_NOT_SET;
-
 	GET_DISK_DESCRIPTOR(fname, ve_private);
 	ret = ploop.read_disk_descr(&di, fname);
 	if (ret) {
@@ -488,7 +472,7 @@ int vzctl_mount_snapshot(unsigned envid, const char *ve_private,
 			goto err;
 		}
 	} else {
-		mount_param.guid = (char *)param->guid;
+		mount_param.guid = param->guid;
 	}
 	mount_param.target = param->target;
 	ploop.set_component_name(di,
@@ -541,12 +525,7 @@ int vzctl_umount_snapshot(unsigned envid, const char *ve_private, char *guid)
 	return 0;
 }
 
-/* Convert a CT to ploop layout
- * 1) mount CT
- * 2) create & mount image
- * 3) cp VE_ROOT -> image
- * 4) update ve_layout
- */
+/* Convert a CT to ploop layout */
 int vzctl_env_convert_ploop(vps_handler *h, envid_t veid,
 		fs_param *fs, dq_param *dq, int mode)
 {
@@ -555,6 +534,7 @@ int vzctl_env_convert_ploop(vps_handler *h, envid_t veid,
 	int ret, ret2;
 	char cmd[STR_SIZE];
 	char new_private[STR_SIZE];
+	char tmp_private[STR_SIZE];
 
 	if (ve_private_is_ploop(fs->private)) {
 		logger(0, 0, "CT is already on ploop");
@@ -575,11 +555,9 @@ int vzctl_env_convert_ploop(vps_handler *h, envid_t veid,
 		logger(-1, 0, "Error: diskspace not set");
 		return VZ_DISKSPACE_NOT_SET;
 	}
-	if (check_ploop_size(dq->diskspace[1]) < 0)
-		return VZ_DISKSPACE_NOT_SET;
 
 	snprintf(new_private, sizeof(new_private), "%s.ploop", fs->private);
-	if (make_dir_mode(new_private, 1, 0600) != 0)
+	if (make_dir_mode(new_private, 1, 0700) != 0)
 		return VZ_CANT_CREATE_DIR;
 
 	param.mode = mode;
@@ -609,14 +587,43 @@ int vzctl_env_convert_ploop(vps_handler *h, envid_t veid,
 		ret = ret2;
 		goto err;
 	}
-	/* Finally, del the old private and replace it with the new one */
-	del_dir(fs->private);
-	rename(new_private, fs->private);
+	/* Move current private to tmp_private */
+	snprintf(tmp_private, sizeof(tmp_private), "%s.XXXXXX", fs->private);
+	if (mkdtemp(tmp_private) == NULL) {
+		logger(-1, errno, "Can't create directory %s", tmp_private);
+		ret = VZ_CANT_CREATE_DIR;
+		goto err;
+	}
+	if (rename(fs->private, tmp_private)) {
+		logger(-1, errno, "Can't rename %s to %s",
+				fs->private, tmp_private);
+		unlink(tmp_private);
+		ret = VZ_SYSTEM_ERROR;
+		goto err;
+	}
+	/* Move new private to its place */
+	if (rename(new_private, fs->private)) {
+		logger(-1, errno, "Can't rename %s to %s",
+				new_private, fs->private);
+
+		/* try to restore old private back */
+		if (rename(tmp_private, fs->private)) {
+			logger(-1, errno, "Can't restore old private area, "
+					"mv %s %s failed, please fix!",
+					tmp_private, fs->private);
+		}
+		ret = VZ_SYSTEM_ERROR;
+		goto err;
+	}
+	/* Fix its mode just in case */
+	chmod(fs->private, 0700);
+	/* Finally, del the old private */
+	destroydir(tmp_private);
 	logger(0, 0, "Container was successfully converted "
 			"to the ploop layout");
 err:
 	if (ret != 0)
-		del_dir(new_private);
+		destroydir(new_private);
 	return ret;
 }
 
