@@ -83,8 +83,8 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 			goto err;
 	} else
 		snprintf(guid, sizeof(guid), "%s", param->guid);
-	GET_SNAPSHOT_XML(fname, fs->private)
-	if (stat_file(fname)) {
+	GET_SNAPSHOT_XML(fname, fs->private);
+	if (stat_file(fname) == 1) {
 		ret = vzctl_read_snapshot_tree(fname, tree);
 		if (ret) {
 			logger(-1, 0, "Failed to read %s", fname);
@@ -146,7 +146,7 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 	// move snapshot.xml to its place
 	GET_SNAPSHOT_XML(fname, fs->private);
 	if (rename(tmp, fname))
-		logger(-1, 0, "Failed to rename %s -> %s", tmp, fname);
+		logger(-1, errno, "Failed to rename %s -> %s", tmp, fname);
 	logger(0, 0, "Snapshot %s has been successfully created",
 			guid);
 	ret = 0;
@@ -154,6 +154,7 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 
 err2:
 	// merge top_delta
+	merge_param.guid = guid;
 	PLOOP_CLEANUP(ret = ploop.merge_snapshot(di, &merge_param));
 	if (ret)
 		logger(-1, 0, "Rollback failed, ploop_merge_snapshot %s: %s [%d]",
@@ -187,16 +188,17 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	char fname[PATH_MAX];
 	char snap_xml_tmp[PATH_MAX];
 	char ve_conf_tmp[PATH_MAX] = "";
-	char topdelta_fname[PATH_MAX] = "";
-	char dd_tmp[PATH_MAX] = "";
 	char dumpfile[PATH_MAX];
+	char guid_buf[39];
+	const char *guid_tmp = NULL;
 	struct vzctl_snapshot_tree *tree = NULL;
 	struct ploop_disk_images_data *di = NULL;
+	struct ploop_snapshot_switch_param param = {};
 
 	if (!is_snapshot_supported(fs->private))
 		return VZCTL_E_SWITCH_SNAPSHOT;
 
-	GET_SNAPSHOT_XML(fname, fs->private)
+	GET_SNAPSHOT_XML(fname, fs->private);
 	if (stat_file(fname) != 1)
 		return vzctl_err(VZCTL_E_SWITCH_SNAPSHOT, 0,
 				"Unable to find snapshot by uuid %s", guid);
@@ -233,10 +235,14 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 		goto err1;
 	/* freeze */
 	if (run) {
-		/* do not destroy current top delta */
+		/* preserve current top delta with 'guid_tmp' for rollback */
+		if (ploop.uuid_generate(guid_buf, sizeof(guid_buf)))
+			return vzctl_err(VZCTL_E_SWITCH_SNAPSHOT, 0,
+					"Can't generate ploop uuid: %s",
+					ploop.get_last_error());
+		guid_tmp = guid_buf;
 		flags = PLOOP_SNAP_SKIP_TOPDELTA_DESTROY;
-		if (ploop.get_top_delta_fname(di, topdelta_fname, sizeof(topdelta_fname)))
-			goto err1;
+
 		ret = vps_chkpnt(h, veid, fs, CMD_SUSPEND, &cpt);
 		if (ret)
 			goto err1;
@@ -244,20 +250,18 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 		if (vps_umount(h, veid, fs, 0))
 			goto err1;
 	}
-	/* switch snapshot */
-	/* preserve DiskDescriptor.xml -> DiskDescriptor.xml.orig */
-	snprintf(dd_tmp, sizeof(dd_tmp), "%s.orig", fname);
-	if (cp_file(dd_tmp, fname))
-		goto err1;
 
-	PLOOP_CLEANUP(ret = ploop.switch_snapshot(di, guid, flags));
-	if (ret) {
-		unlink(dd_tmp);
+	/* switch snapshot */
+	param.guid = guid;
+	param.guid_old = guid_tmp;
+	param.flags = flags;
+	PLOOP_CLEANUP(ret = ploop.switch_snapshot_ex(di, &param));
+	if (ret)
 		goto err2;
-	}
+
 	/* restore ve.conf */
 	vzctl_get_snapshot_ve_conf(fs->private, guid, fname, sizeof(fname));
-	if (stat_file(fname)) {
+	if (stat_file(fname) == 1) {
 		get_vps_conf_path(veid, ve_conf_tmp, sizeof(ve_conf_tmp) - 4);
 		strcat(ve_conf_tmp, ".tmp");
 		if (cp_file(ve_conf_tmp, fname))
@@ -280,25 +284,23 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	}
 	/* resume CT in case dump file exists (no rollback, ignore error) */
 	vzctl_get_snapshot_dumpfile(fs->private, guid, dumpfile, sizeof(dumpfile));
-	if (stat_file(dumpfile)) {
+	if (stat_file(dumpfile) == 1) {
 		vps_param *param;
 
 		param = reread_vps_config(veid);
 		cpt.dumpfile = dumpfile;
 		if (vps_restore(h, veid, (param) ? param : g_p,
-					CMD_RESTORE, &cpt, SKIP_NONE))
+					CMD_RESTORE, &cpt, SKIP_DUMPFILE_UNLINK))
 			logger(-1, 0, "Failed to resume Container");
 		free_vps_param(param);
 	}
 	GET_SNAPSHOT_XML(fname, fs->private);
 	if (rename(snap_xml_tmp, fname))
-		logger(-1, 0, "Failed to rename %s %s", snap_xml_tmp, fname);
+		logger(-1, errno, "Failed to rename %s %s", snap_xml_tmp, fname);
 
-	/* cleanup */
-	if (dd_tmp[0] != '\0' && unlink(dd_tmp))
-		logger(-1, errno, "Failed to unlink %s", dd_tmp);
-	if (topdelta_fname[0] != '\0' && unlink(topdelta_fname))
-		logger(-1, errno, "Failed to unlink %s", topdelta_fname);
+	/* remove temporary snapshot */
+	if (guid_tmp != NULL)
+		vzctl_delete_snapshot(fs->private, guid_tmp);
 
 	logger(0, 0, "Container has been successfully switched "
 			"to another snapshot");
@@ -306,21 +308,17 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	goto out;
 
 err3:
-	/* Restore original DiskDescriptor.xml */
-	GET_DISK_DESCRIPTOR(fname, fs->private);
-	if (rename(dd_tmp, fname)) {
-		logger(-1, errno, "Failed to rename %s -> %s",
-				dd_tmp, fname);
-		goto err2;
+	if (guid_tmp != NULL) {
+		param.guid = guid_tmp;
+		param.guid_old = NULL;
+		param.flags = PLOOP_SNAP_SKIP_TOPDELTA_CREATE;
+		PLOOP_CLEANUP(ploop.switch_snapshot_ex(di, &param));
 	}
-	/* remove new top delta */
-	if (ploop.get_top_delta_fname(di, topdelta_fname, sizeof(topdelta_fname)) == 0)
-		if (unlink(topdelta_fname))
-			logger(-1, errno, "Failed to unlink %s", topdelta_fname);
 
 err2:
-	if (cpt_cmd(h, veid, fs->root, CMD_CHKPNT, CMD_RESUME, 0))
+	if (run && cpt_cmd(h, veid, fs->root, CMD_CHKPNT, CMD_RESUME, 0))
 		logger(-1, 0, "Failed to resume Container on error");
+
 err1:
 	unlink(snap_xml_tmp);
 
@@ -348,7 +346,7 @@ int vzctl_env_delete_snapshot(vps_handler *h, envid_t veid,
 	if (!is_snapshot_supported(fs->private))
 		return VZCTL_E_DELETE_SNAPSHOT;
 
-	GET_SNAPSHOT_XML(fname, fs->private)
+	GET_SNAPSHOT_XML(fname, fs->private);
 	if (stat_file(fname) != 1)
 		return vzctl_err(VZCTL_E_DELETE_SNAPSHOT, 0,
 				"Unable to find snapshot by uuid %s", guid);
@@ -379,7 +377,7 @@ int vzctl_env_delete_snapshot(vps_handler *h, envid_t veid,
 		goto err1;
 
 	vzctl_get_snapshot_dumpfile(fs->private, guid, fname, sizeof(fname));
-	if (stat_file(fname)) {
+	if (stat_file(fname) == 1) {
 		logger(1, 0, "Deleting CT dump %s", fname);
 		if (unlink(fname))
 			logger(-1, errno, "Failed to delete dump %s",
@@ -388,13 +386,13 @@ int vzctl_env_delete_snapshot(vps_handler *h, envid_t veid,
 
 	// delete ve.conf
 	vzctl_get_snapshot_ve_conf(fs->private, guid, fname, sizeof(fname));
-	if (stat_file(fname) && unlink(fname))
+	if (stat_file(fname) == 1 && unlink(fname))
 		logger(-1, errno, "Failed to delete %s", fname);
 
 	// move snapshot.xml to its place
 	GET_SNAPSHOT_XML(fname, fs->private);
 	if (rename(tmp, fname))
-		logger(-1, 0, "Failed to rename %s %s", tmp, fname);
+		logger(-1, errno, "Failed to rename %s %s", tmp, fname);
 
 	logger(0, 0, "Snapshot %s has been successfully deleted", guid);
 	vzctl_free_snapshot_tree(tree);
@@ -414,10 +412,6 @@ int vzctl_env_mount_snapshot(unsigned veid, char *ve_private, char *mnt, char *g
 	struct vzctl_snapshot_tree *tree = NULL;
 	struct vzctl_mount_param param = {};
 
-	if (guid == NULL)
-		return vzctl_err(VZ_INVALID_PARAMETER_SYNTAX, 0,
-				"Failed to mount snapshot: "
-				"snapshot uuid not specified");
 	if (ve_private == NULL)
 		return vzctl_err(VZ_VE_PRIVATE_NOTSET, 0,
 				"Failed to mount snapshot: "
@@ -462,10 +456,6 @@ free_tree:
 
 int vzctl_env_umount_snapshot(unsigned veid, char *ve_private, char *guid)
 {
-	if (guid == NULL)
-		return vzctl_err(VZ_INVALID_PARAMETER_SYNTAX, 0,
-				"Failed to umount snapshot: "
-				"snapshot uuid not specified");
 	if (ve_private == NULL)
 		return vzctl_err(VZ_VE_PRIVATE_NOTSET, 0,
 				"Failed to umount snapshot: "
