@@ -148,6 +148,8 @@ static struct option set_opt[] = {
 	{"ram",		required_argument, NULL, PARAM_RAM},
 	{"swap",	required_argument, NULL, PARAM_SWAP},
 
+	{"stop-timeout", required_argument, NULL, PARAM_STOP_TIMEOUT},
+
 	{NULL, 0, NULL, 0}
 };
 
@@ -221,6 +223,7 @@ static int parse_startstop_opt(int argc, char **argv, vps_param *param,
 		{"wait", no_argument, NULL, PARAM_WAIT},
 		{"fast", no_argument, NULL, PARAM_FAST},
 		{"skip-umount", no_argument, NULL, PARAM_SKIP_UMOUNT},
+		{"skip-fsck", no_argument, NULL, PARAM_SKIP_FSCK},
 		{ NULL, 0, NULL, 0 }
 	};
 
@@ -259,6 +262,12 @@ static int parse_startstop_opt(int argc, char **argv, vps_param *param,
 			else
 				goto err;
 			break;
+		case PARAM_SKIP_FSCK:
+			if (start)
+				param->opt.skip_fsck = YES;
+			else
+				goto err;
+			break;
 		default:
 			ret = VZ_INVALID_PARAMETER_SYNTAX;
 			break;
@@ -279,7 +288,7 @@ err:
  * Otherwise return -1.
  */
 static int try_restore(vps_handler *h, envid_t veid, vps_param *g_p,
-		vps_param *cmd_p)
+		vps_param *cmd_p, skipFlags skip)
 {
 	char buf[STR_SIZE];
 	char *dumpdir = g_p->res.cpt.dumpdir;
@@ -294,7 +303,7 @@ static int try_restore(vps_handler *h, envid_t veid, vps_param *g_p,
 	cmd_p->res.cpt.dumpdir = dumpdir;
 
 	return vps_restore(h, veid, g_p, CMD_RESTORE, &cmd_p->res.cpt,
-			SKIP_UMOUNT);
+			skip | SKIP_UMOUNT);
 }
 
 static int start(vps_handler *h, envid_t veid, vps_param *g_p,
@@ -324,14 +333,17 @@ static int start(vps_handler *h, envid_t veid, vps_param *g_p,
 		logger(-1, 0, "Container is already running");
 		return VZ_VE_RUNNING;
 	}
+	/* Set skip flags */
+	if (cmd_p->opt.skip_setup == YES)
+		skip |= SKIP_SETUP;
+	if (cmd_p->opt.skip_fsck == YES)
+		skip |= SKIP_FSCK;
 	/* Try restore first */
-	ret = try_restore(h, veid, g_p, cmd_p);
+	ret = try_restore(h, veid, g_p, cmd_p, skip);
 	if (ret == 0)
 		return ret;
 	if (ret != -1) /* We tried to restore */
 		skip |= SKIP_REMOUNT; /* Do not remount just mounted fs */
-	if (cmd_p->opt.skip_setup == YES)
-		skip |= SKIP_SETUP;
 	g_p->res.misc.wait = cmd_p->res.misc.wait;
 	ret = vps_start(h, veid, g_p, skip, &g_action);
 
@@ -904,6 +916,11 @@ static int check_set_opt(int argc, char *argv[], vps_param *param)
 				 " --name option without --save");
 			return VZ_SET_NAME_ERROR;
 		}
+		if (param->res.misc.stop_timeout >= 0) {
+			logger(-1, 0, "Error: unable to use"
+				" --stop-timeout option without --save");
+			return VZ_INVALID_PARAMETER_SYNTAX;
+		}
 		if (param->opt.save_force) {
 			logger(-1, 0, "Error: --force is useless "
 					"without --save");
